/**
 * Page Loading Progress Bar.
 * Exclusively on Envato Market: https://1.envato.market/laser
 *
 * @encoding        UTF-8
 * @version         1.0.0
 * @copyright       Copyright (C) 2018 - 2020 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/
/* Based on https://github.com/rstacruz/nprogress */

"use strict";

;(function (root, factory) {

    if ( typeof define === 'function' && define.amd ) {
        define( factory );
    } else if ( typeof exports === 'object' ) {
        module.exports = factory();
    } else {
        root.mdpLaserPlugin = factory();
    }

} )( this, function () {
    let mdpLaserPlugin = {};

    mdpLaserPlugin.version = '1.0.0';

    const mdpLaserOptions = window.mdpLaserWP;

    let Settings = mdpLaserPlugin.settings = {
        minimum: 0.08,
        easing: mdpLaserOptions.easing,
        positionUsing: '',
        speed: mdpLaserOptions.speed,
        trickle: ( mdpLaserOptions.trickle === 'on' ),
        trickleSpeed: mdpLaserOptions.speed,
        barSelector: '.mdp-laser',
        parent: 'body',
        template: '<div class="mdp-laser"></div>'
    };

    /**
     * Updates configuration.
     *
     *     mdpLaserPlugin.configure({
     *       minimum: 0.1
     *     });
     */
    mdpLaserPlugin.configure = function (options) {
        let key, value;
        for (key in options) {
            // noinspection JSUnfilteredForInLoop
            value = options[key];
            if (value !== undefined && options.hasOwnProperty(key)) Settings[key] = value;
        }

        return this;
    };

    /**
     * Last number.
     */

    mdpLaserPlugin.status = null;

    /**
     * Sets the progress bar status, where `n` is a number from `0.0` to `1.0`.
     *
     *     mdpLaserPlugin.set(0.4);
     *     mdpLaserPlugin.set(1.0);
     **/
    mdpLaserPlugin.set = function (n) {
        let started = mdpLaserPlugin.isStarted();

        n = clamp(n, Settings.minimum, 1);

        mdpLaserPlugin.status = (n === 1 ? null : n);

        let progress = mdpLaserPlugin.render(!started),
            bar = progress.querySelector(Settings.barSelector),
            speed = Settings.speed,
            ease = Settings.easing;

        progress.offsetWidth; /* Repaint */

        queue(function (next) {
            // Set positionUsing if it hasn't already been set
            if (Settings.positionUsing === '') Settings.positionUsing = mdpLaserPlugin.getPositioningCSS();

            // Add transition
            css(bar, barPositionCSS(n, speed, ease));

            if (n === 1) {
                // Fade out
                css(progress, {
                    transition: 'none',
                    opacity: 1
                });
                progress.offsetWidth; /* Repaint */

                setTimeout(function () {
                    css(progress, {
                        transition: 'all ' + speed + 'ms linear',
                        opacity: 0
                    });
                    setTimeout(function () {
                        mdpLaserPlugin.remove();
                        next();
                    }, speed);
                }, speed);
            } else {
                setTimeout(next, speed);
            }
        });

        return this;
    };

    mdpLaserPlugin.isStarted = function () {
        return typeof mdpLaserPlugin.status === 'number';
    };

    /**
     * Shows the progress bar.
     * This is the same as setting the status to 0%, except that it doesn't go backwards.
     *
     *     mdpLaserPlugin.start();
     *
     **/
    mdpLaserPlugin.start = function () {

        if ( ! mdpLaserPlugin.status ) {

            mdpLaserPlugin.set( 0 );

        }

        let work = function () {

            setTimeout( function () {

                if ( ! mdpLaserPlugin.status ) { return; }

                mdpLaserPlugin.trickle();
                localStorage.setItem( 'mdpLaserStatus', mdpLaserPlugin.status );
                work();

            }, Settings.trickleSpeed );

        };

        if ( Settings.trickle ) { work(); }

        return this;

    };

    /**
     * Hides the progress bar.
     * This is the *sort of* the same as setting the status to 100%, with the
     * difference being `done()` makes some placebo effect of some realistic motion.
     *
     *     mdpLaserPlugin.done();
     *
     * If `true` is passed, it will show the progress bar even if its hidden.
     *
     *     mdpLaserPlugin.done(true);
     */

    mdpLaserPlugin.done = function (force) {
        if (!force && !mdpLaserPlugin.status) return this;

        return mdpLaserPlugin.inc(0.3 + 0.5 * Math.random()).set(1);
    };

    /**
     * Increments by a random amount.
     **/
    mdpLaserPlugin.inc = function (amount) {
        let n = mdpLaserPlugin.status;

        if (!n) {
            return mdpLaserPlugin.start();
        } else if (n > 1) {
            // noinspection UnnecessaryReturnStatementJS
            return;
        } else {
            if (typeof amount !== 'number') {
                if (n >= 0 && n < 0.2) {
                    amount = 0.1;
                } else if (n >= 0.2 && n < 0.5) {
                    amount = 0.04;
                } else if (n >= 0.5 && n < 0.8) {
                    amount = 0.02;
                } else if (n >= 0.8 && n < 0.99) {
                    amount = 0.005;
                } else {
                    amount = 0;
                }
            }

            n = clamp(n + amount, 0, 0.994);

            return mdpLaserPlugin.set(n);
        }
    };

    mdpLaserPlugin.trickle = function () {
        return mdpLaserPlugin.inc();
    };

    /**
     * Waits for all supplied jQuery promises and
     * increases the progress as the promises resolve.
     *
     * @param $promise jQuery Promise
     **/
    (function () {
        let initial = 0, current = 0;

        mdpLaserPlugin.promise = function ($promise) {
            if (!$promise || $promise.state() === "resolved") {
                return this;
            }

            if (current === 0) {
                mdpLaserPlugin.start();
            }

            initial++;
            current++;

            $promise.always(function () {
                current--;
                if (current === 0) {
                    initial = 0;
                    mdpLaserPlugin.done();
                } else {
                    mdpLaserPlugin.set((initial - current) / initial);
                }
            });

            return this;
        };

    })();

    /**
     * (Internal) renders the progress bar markup based on the `template`
     * setting.
     **/
    mdpLaserPlugin.render = function ( fromStart ) {

        if ( mdpLaserPlugin.isRendered() ) {
            return document.getElementById( 'mdp-laser' );
        }

        addClass( document.documentElement, 'mdp-laser-busy' );

        let progress = document.createElement('div');
        progress.id = 'mdp-laser';
        progress.innerHTML = Settings.template;

        let bar = progress.querySelector( Settings.barSelector );
        let parent = document.querySelector( Settings.parent );
        let perc;

        /** Reverse Direction. */
        if ( 'on' === mdpLaserOptions.reverse ) {
            perc = fromStart ? '100' : toBarPerc( mdpLaserPlugin.status || 0 );

        /** Normal Direction. */
        } else {
            perc = fromStart ? '-100' : toBarPerc( mdpLaserPlugin.status || 0 );
        }

        /** Set progress styles */
        progress.style.height = mdpLaserOptions.height + 'px';
        progress.style.position = 'fixed';
        progress.style.zIndex = '99999';
        progress.style.backgroundColor = mdpLaserOptions.bgColor;

        /** Set bar styles. */
        bar.style.height = mdpLaserOptions.height + 'px';
        bar.style.position = 'fixed';
        bar.style.left = 0;
        bar.style.width = '100%';

        /** Style */
        if ( typeof mdpLaserOptions.style === "string" ) {

            bar.style.backgroundColor = mdpLaserOptions.color;

            if ( mdpLaserOptions.style === 'gradient' ) {

                bar.style.backgroundImage = 'linear-gradient( to left, ' + mdpLaserOptions.color + ', ' + mdpLaserOptions.gradient + ' )';

            }

            if ( mdpLaserOptions.style === 'pulse' ) {

                let mdpLaserStyle = '.mdp-laser:after { content: ""; opacity: 0; position: absolute; top: 0; right: 0; bottom: 0; left: 0; background: #fff; animation: mdp-laser-pulse 1s linear infinite; }';
                mdpLaserStyle += '@keyframes mdp-laser-pulse { 0% { opacity: 0; width: 0; } 50% { opacity: .4; } 100% { opacity: 0; width: 98%; } }';

                let pageHead = document.head || document.getElementsByTagName('head')[0],
                    pageStyle = document.createElement('style');

                pageHead.appendChild( pageStyle );
                pageStyle.appendChild( document.createTextNode( mdpLaserStyle ) );

            }

        }

        /** Reverse Direction. */
        if ( 'on' === mdpLaserOptions.reverse ) {

            bar.style.left = 'auto !important';
            bar.style.right = 0;

        }

        /** Laser position */
        if ( typeof mdpLaserOptions.position === "string" ) {

            if ( 'bottom' === mdpLaserOptions.position ) {

                progress.style.width = '100wh';
                progress.style.bottom = '0';

            }

            if ( 'top' === mdpLaserOptions.position ) {

                progress.style.width = '100wh';
                progress.style.top = '0';

            }

            if ( 'left' === mdpLaserOptions.position ) {

                progress.style.width = '100vh';
                progress.style.top = '0';
                progress.style.left = mdpLaserOptions.height + 'px';
                progress.style.transform = 'rotate( 90deg )';
                progress.style.transformOrigin = 'left top';

            }

            if ( 'right' === mdpLaserOptions.position ) {

                progress.style.width = '100vh';
                progress.style.bottom = '0';
                progress.style.right = mdpLaserOptions.height + 'px';
                progress.style.transform = 'rotate( 90deg )';
                progress.style.transformOrigin = 'right bottom';

            }

        }

        /** Box shadow */
        if ( typeof mdpLaserOptions.boxShadow === "string" ) {

            if ( 'on' === mdpLaserOptions.boxShadow ) {

                bar.style.boxShadow = '0 0 10px ' + mdpLaserOptions.color + ', 0 0 5px ' + mdpLaserOptions.color;

            }

        }

        css( bar, {
            transition: 'all 0 linear',
            transform: 'translate3d(' + perc + '%,0,0)'
        } );

        if ( parent !== document.body ) {
            addClass( parent, 'mdp-laser-custom-parent' );
        }

        parent.appendChild( progress );
        return progress;
    };

    /**
     * Removes the element. Opposite of render().
     **/
    mdpLaserPlugin.remove = function () {
        removeClass( document.documentElement, 'mdp-laser-busy' );
        removeClass( document.querySelector( Settings.parent ), 'mdp-laser-custom-parent' );
        let progress = document.getElementById( 'mdp-laser' );
        progress && removeElement( progress );
    };

    /**
     * Checks if the progress bar is rendered.
     **/
    mdpLaserPlugin.isRendered = function () {
        return !!document.getElementById( 'mdp-laser' );
    };

    /**
     * Determine which positioning CSS rule to use.
     */
    mdpLaserPlugin.getPositioningCSS = function () {
        // Sniff on document.body.style
        let bodyStyle = document.body.style;

        // Sniff prefixes
        let vendorPrefix = ('WebkitTransform' in bodyStyle) ? 'Webkit' :
            ('MozTransform' in bodyStyle) ? 'Moz' :
                ('msTransform' in bodyStyle) ? 'ms' :
                    ('OTransform' in bodyStyle) ? 'O' : '';

        if (vendorPrefix + 'Perspective' in bodyStyle) {
            // Modern browsers with 3D support, e.g. Webkit, IE10
            return 'translate3d';
        } else if (vendorPrefix + 'Transform' in bodyStyle) {
            // Browsers without 3D support, e.g. IE9
            return 'translate';
        } else {
            // Browsers without translate() support, e.g. IE7-8
            return 'margin';
        }
    };

    /**
     * Helpers
     **/
    function clamp(n, min, max) {

        if ( n < min ) { return min; }

        if ( n > max ) { return max; }

        return n;
    }

    /**
     * (Internal) converts a percentage (`0..1`) to a bar translateX
     * percentage (`-100%..0%`).
     **/
    function toBarPerc(n) {

        /** Reverse Direction. */
        if ( 'on' === mdpLaserOptions.reverse ) {
            return 100 - (n * 100);

        /** Normal Direction. */
        } else {
            return (-1 + n) * 100;
        }

    }


    /**
     * (Internal) returns the correct CSS for changing the bar's
     * position given an n percentage, and speed and ease from Settings
     */
    function barPositionCSS(n, speed, ease) {
        let barCSS;

        if (Settings.positionUsing === 'translate3d') {
            barCSS = {transform: 'translate3d(' + toBarPerc(n) + '%,0,0)'};
        } else if (Settings.positionUsing === 'translate') {
            barCSS = {transform: 'translate(' + toBarPerc(n) + '%,0)'};
        } else {
            barCSS = {'margin-left': toBarPerc(n) + '%'};
        }

        barCSS.transition = 'all ' + speed + 'ms ' + ease;

        return barCSS;
    }

    /**
     * (Internal) Queues a function to be executed.
     */
    let queue = (function () {
        let pending = [];

        function next() {
            let fn = pending.shift();
            if (fn) {
                fn(next);
            }
        }

        return function (fn) {
            pending.push(fn);
            if (pending.length === 1) next();
        };
    })();

    /**
     * (Internal) Applies css properties to an element, similar to the jQuery
     * css method.
     *
     * While this helper does assist with vendor prefixed property names, it
     * does not perform any manipulation of values prior to setting styles.
     */
    let css = (function () {
        let cssPrefixes = ['Webkit', 'O', 'Moz', 'ms'],
            cssProps = {};

        function camelCase(string) {
            return string.replace(/^-ms-/, 'ms-').replace(/-([\da-z])/gi, function (match, letter) {
                return letter.toUpperCase();
            });
        }

        function getVendorProp(name) {
            let style = document.body.style;
            if (name in style) return name;

            let i = cssPrefixes.length,
                capName = name.charAt(0).toUpperCase() + name.slice(1),
                vendorName;
            while (i--) {
                vendorName = cssPrefixes[i] + capName;
                if (vendorName in style) return vendorName;
            }

            return name;
        }

        function getStyleProp(name) {
            name = camelCase(name);
            return cssProps[name] || (cssProps[name] = getVendorProp(name));
        }

        function applyCss(element, prop, value) {
            prop = getStyleProp(prop);
            element.style[prop] = value;
        }

        return function (element, properties) {
            let args = arguments,
                prop,
                value;

            if (args.length === 2) {
                for (prop in properties) {
                    // noinspection JSUnfilteredForInLoop
                    value = properties[prop];
                    if (value !== undefined && properties.hasOwnProperty(prop)) applyCss(element, prop, value);
                }
            } else {
                applyCss(element, args[1], args[2]);
            }
        }
    })();

    /**
     * (Internal) Determines if an element or space separated list of class names contains a class name.
     */
    function hasClass(element, name) {
        let list = typeof element == 'string' ? element : classList(element);
        return list.indexOf(' ' + name + ' ') >= 0;
    }

    /**
     * (Internal) Adds a class to an element.
     */
    function addClass(element, name) {
        let oldList = classList(element),
            newList = oldList + name;

        if (hasClass(oldList, name)) return;

        // Trim the opening space.
        element.className = newList.substring(1);
    }

    /**
     * (Internal) Removes a class from an element.
     */
    function removeClass(element, name) {
        let oldList = classList(element),
            newList;

        if (!hasClass(element, name)) return;

        // Replace the class name.
        newList = oldList.replace(' ' + name + ' ', ' ');

        // Trim the opening and closing spaces.
        element.className = newList.substring(1, newList.length - 1);
    }

    /**
     * (Internal) Gets a space separated list of the class names on the element.
     * The list is wrapped with a single space on each end to facilitate finding
     * matches within the list.
     */
    function classList( element ) {
        return (' ' + ( element && element.className || '') + ' ').replace(/\s+/gi, ' ');
    }

    /**
     * (Internal) Removes an element from the DOM.
     **/
    function removeElement( element ) {
        element && element.parentNode && element.parentNode.removeChild( element );
    }

    return mdpLaserPlugin;
} );

window.addEventListener('beforeunload', function () {

    mdpLaserPlugin.start();

} );

document.addEventListener( 'DOMContentLoaded', function() {

    /** Read previously-stored progress. */
    let progress = localStorage.getItem( 'mdpLaserStatus' );
    progress = parseFloat( progress );

    /** Set loader to progress. */
    mdpLaserPlugin.set( progress );

    /** Run progress. */
    mdpLaserPlugin.start();

} );

window.addEventListener( 'load', function () {

    mdpLaserPlugin.done( true );

} );
